/******* 040020337.cpp ****************************************
This program finds the triangle : which is represented by three
lines that are the tangents of the given circle on given points

Programmer : Beycan Kahraman                  Date : 21.03.2005 
**************************************************************/

#include <iostream> // necessary library for cin, cout objects
using namespace std; // setting default namespace

/*== LINE ===================================================*/
class Line{
    double x, y, m;                   // coordinates of the line
    public:
        Line(double = 0, double = 0, double = 10000000); // constructor
        double getX();                // returns the x
        double getY();                // returns the y
        double getM();                // returns the slope
        bool onLine(double, double);
        void show();                  // prints the line
};
Line::Line(double X, double Y, double M){ // constructing line
    x = X;
    y = Y;
    m = M;
}
double Line::getX(){ // returns the x
    return x;
}  
double Line::getY(){ // returns the y
    return y;
}
double Line::getM(){ // returns the slope
    return m;
}  
bool Line::onLine(double X, double Y){ // controls if the point is on the Line
    return fabs(m * (x - X) - Y + y) < 0.0001;
}
void Line::show(){ // prints the line
    if(fabs(m) < 10000000) // for non-vertical lines
        cout << "LINE    (x, y, m) =  ( " << x << ", " << y << ", " << m << " )" << endl; 
    else // for vertical lines
        cout << "LINE    (x, y, m) =  ( " << x << ", " << y << ", " << "infinity )" << endl; 
}

/*== CIRCLE =================================================*/
class Circle{
    double x, y, r;                      // coordinates of the circle
    public:
        Circle(double = 0, double = 0, double = 10); // constructor
        Line getTangent(double, double); // returns the tangent line at (x, y)
        bool onCircle(double, double);
        void show();                     // prints the circle
        void set(double=0, double=0, double = 10); // setting new values
};
Circle::Circle(double X, double Y, double R){ // constructing circle
    x = X;
    y = Y;
    r = R;
    if(r < 0.0001) // for unavailable diameters
    {
        r = 10;
        cout << ":: WARNING ::     r is unavailable.    ( We choose   r = 10 )" << endl;
    }    
}    
Line Circle::getTangent(double X, double Y){ // returns the tangent line at (x, y)
    double enlarge; // if the point is not on the circle, we will enlarge it
    if(onCircle(X,Y)) // if point is on circle
    {
        if(fabs(y - Y) < 0.0001)
            return Line(X, Y); // returning vertical line
        return Line(X, Y, (x - X) / (Y - y)); // returning non-vertical line
    }
    else // the point is not on circle
    {
        if(fabs(pow(x - X, 2) + pow(y - Y, 2)) < 0.000001) // Point is on the Center
        {
            X = X + 1;
            Y = Y + 1;
            cout << ":: WARNING ::     Point is on the Center, Moved to (X++, Y++)" << endl;
        }
        enlarge = sqrt(pow(x - X, 2) + pow(y - Y, 2)) / r;
        X = (X - x) / enlarge + x;
        Y = (Y - y) / enlarge + y;
        cout << ":: WARNING ::     Point is not on the Circle, Moved to (" << X << "," << Y << ")" << endl;
        // if the point is not on the Circle, We Move it to the nearest point on the Circle
    }    
    if(fabs(y - Y) < 0.0001)
        return Line(X, Y); // returning vertical line
    return Line(X, Y, (x - X) / (Y - y)); // returning non-vertical line
} 
bool Circle::onCircle(double X, double Y){ // controls if the point is on the Circle
    return fabs( pow(x - X, 2) + pow(y - Y, 2) - pow(r, 2) ) < 0.0001;
}    
void Circle::show(){ // prints the circle
    cout << "CIRCLE  (x, y, r) =  ( " << x << ", " << y << ", " << r << " )" << endl;
}   
void Circle::set(double X,double Y,double R){ // setting new values of the circle
    x = X;
    y = Y;
    r = R;
    if(r < 0.0001){ // for unavailable diameters
        r = 10;
        cout << ":: WARNING ::     r is unavailable.    ( We choose   r = 10 )" << endl;
    }
}

/*== TRIANGLE ===============================================*/
class Triangle{
    double x[3], y[3];      // coordinates of the triangle
    bool possible;          // controlling if there are any parallel lines
    public:
        Triangle(Line L[]); // constructor
        void getCorners();  // prints the corner coordinates of the triangle
        void set(Line L[]); // setting the new values of the triangle
};
Triangle::Triangle(Line L[]){        // constructing the triangle
    possible = true;
    if(L[0].getM() == L[1].getM() || // controlling if there are any parallel lines
       L[1].getM() == L[2].getM() || 
       L[2].getM() == L[0].getM())
        possible = false;            // there are parallel lines
    for(int i = 0; i < 3; i++)
    {   // calculating the corner points of the triangle
        x[i] = (L[(i + 1)%3].getY() - L[i].getY() + L[i].getM()*L[i].getX() - L[(i + 1)%3].getM()*L[(i + 1)%3].getX())
                / (L[i].getM() - L[(i + 1)%3].getM());
        y[i] = L[i].getM() * (x[i] - L[i].getX()) + L[i].getY();
    }
}  
void Triangle::getCorners(){ // printing the corner points of the triangle
    if(possible)
        cout << "TRIANGLE(P, Q, R) =  ((" << x[0] << "," << y[0] << "), (" << x[1] << "," << y[1] << "), (" << x[2] << "," << y[2] << "))" << endl;
    else
        cout << ":: ERROR ::       There are Tangents With Equal Slopes." << endl;
}
void Triangle::set(Line L[]){        // setting the new values of the triangle
    possible=true;
    if(L[0].getM() == L[1].getM() || // controlling if there are any parallel lines
       L[1].getM() == L[2].getM() || 
       L[2].getM() == L[0].getM())
        possible=false;              // there are parallel lines
    for(int i=0;i<3;i++)
    {   // calculating the corner points of the triangle
        x[i] = (L[(i + 1)%3].getY() - L[i].getY() + L[i].getM()*L[i].getX() - L[(i + 1)%3].getM()*L[(i + 1)%3].getX())
                / (L[i].getM() - L[(i + 1)%3].getM());
        y[i] = L[i].getM() * (x[i] - L[i].getX()) + L[i].getY();
    }
}

/*== MAIN PROGRAM ===========================================*/
int main(void){
    
    double number[3]; // these numbers are used to take the inputs
    Circle circle(0, 0, 10); // circle is constructed here
    cout << "==================================================================" << endl;
    circle.show();
    Line line[]={circle.getTangent(6, 8), circle.getTangent(-8, 6), circle.getTangent(0, -10)};
    for(int i = 0; i < 3; i++)
        line[i].show();
    Triangle triangle(line);
    triangle.getCorners();
    
    cout << "==================================================================" << endl;
    while(true){
        cout << endl << "==>  Do you want to set your values? (1/0) ";
        cin >> number[0];
        cout << "==================================================================" << endl;
        if(number[0] == 0)
            break;
            
        cout << "Circle  (x, y, r) = ";
        cin >> number[0] >> number[1] >> number[2];
        circle.set(number[0], number[1], number[2]);
        for(int i = 0; i < 3; i++){
            cout << "Tangent " << i+1 << "  (x, y) = ";
            cin >> number[0] >> number[1];
            line[i] = circle.getTangent(number[0], number[1]);
        }
        
        cout << "------------------------------------------------------------------" << endl;
        circle.show();
        for(int i = 0; i < 3; i++)
            line[i].show();
        triangle.set(line);
        triangle.getCorners();
        cout << "==================================================================" << endl;
    }
    
    system("PAUSE");
    return 1;
}

